require 'rlml'

module RLML
  MARSHAL_PREFIX = "ruby."

  def self.dump(object)
    _rlml_dump(object).to_s
  end

  def self.load(string)
    _rlml_load(TreeParser.new.parse(string)[0])
  end

  private
  def self._rlml_dump(object)
    # Don't pollute core classes
    if object.respond_to? :_dump_rlml
      _dump_rlml object
    else
      args = case object
             when String, Numeric, Regexp
               # This sucks
               [object.to_s]
             when Array
               object.map { |o| _rlml_dump o }
             when Range
               [_rlml_dump(object.begin), 
                _rlml_dump(object.end), 
                _rlml_dump(object.exclude_end?)] 
             when Hash
               object.inject([]) { |r,(k,v)| r <<_rlml_dump(k) <<_rlml_dump(v) }
             when Time
               object.to_i.to_s
             when TrueClass, FalseClass, NilClass
               []
             else
               raise TypeError,
                     "no _dump_rlml is defined for class #{object.class}"
             end

      Tree.new(MARSHAL_PREFIX + object.class.to_s, 'utf-8', *args)
    end
  end

  def self._rlml_load(tree)
    klassname = tree.name.gsub(/^#{Regexp.quote MARSHAL_PREFIX}/, '')

    begin
      klass = Object.const_get klassname
    rescue NameError
      raise ArgumentError, "undefined class/module #{klassname}"
    end

    if klass.respond_to? :_load_rlml
      _load_rlml tree
    else
      # case..end won't work here :(
      if klass <= String
        tree[0]
      elsif klass <= Regexp
        Regexp.new tree[0]
      elsif klass <= Integer
        Integer(tree[0])
      elsif klass <= Float
        Float(tree[0])
      elsif klass <= Array
        tree.children.map { |o| _rlml_load o }
      elsif klass <= Hash
        Hash[*tree.children.map { |o| _rlml_load o }]
      elsif klass <= Time
        Time.at Integer(tree[0])
      else
        raise ArgumentError, "can't load #{klassname}"        
      end
    end
  end
end

if $0 == __FILE__
$VERBOSE = true
puts RLML.dump(2)

puts RLML.dump('foo')
puts RLML.dump("fo\no")

puts RLML.dump([1,2,3])

puts RLML.dump({1=>2,2=>3})

puts RLML.dump(Time.now)

puts RLML.dump(3.4)

puts RLML.dump(1..4)
puts RLML.dump(1...4)

puts RLML.dump(/foo/)

begin
  puts RLML.dump(lambda {})
rescue
  p $!
  ;
end

p RLML.load('rlml/utf-8 ruby.Float{"3.4"}')

p RLML.load(RLML.dump([1,2,3]))

p RLML.load(RLML.dump({1=>2,2=>3}))

p RLML.load(RLML.dump(Time.now))
end

# arch-tag: be30beaf-3392-4402-b9f5-2791693cfe5b
